using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinDataSource;
using Infragistics.Win.UltraWinGrid;
using ColumnHeader=Infragistics.Win.UltraWinGrid.ColumnHeader;

namespace EditAsTime
{
    public partial class Form1 : Form
    {
        private readonly List<DateTime> data = new List<DateTime>();
        private bool isFirstClickPerformed = false;
        private string clickedColumnHeader_;

        /// <summary>
        /// Gets or sets the clicked column header.
        /// </summary>
        /// <value>The clicked column header.</value>
        public string ClickedColumnHeader
        {
            get { return clickedColumnHeader_; }
            set { clickedColumnHeader_ = value; }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();
            grid_.DisplayLayout.Override.CellClickAction = CellClickAction.RowSelect;

            BuildTestData();
            PopulateDataSource();
        }

        /// <summary>
        /// Builds the test data.
        /// </summary>
        private void BuildTestData()
        {
            data.Add(new DateTime(1900, 1, 1, 8, 0, 0));
            data.Add(new DateTime(1922, 1, 1, 12, 0, 0));
            data.Add(new DateTime(1944, 1, 1, 16, 0, 0));
        }

        /// <summary>
        /// Populates the data source.
        /// </summary>
        private void PopulateDataSource()
        {
            int i = 5;
            foreach (DateTime time in data)
            {
                UltraDataRow row = dataSource_.Rows.Add();
                row["Start"] = time;
                row["End"] = time;
                row["Value"] = i++;
            }
        }

        /// <summary>
        /// Called when [grid mouse click].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        private void OnGridMouseClick(object sender, MouseEventArgs e)
        {
            UltraGrid grid = (UltraGrid) sender;
            SortIndicator valueIndicator = grid_.DisplayLayout.Bands[0].Columns["Value"].SortIndicator;
            if (grid != null)
            {
                UIElement element = grid.DisplayLayout.UIElement.LastElementEntered;
                if (element != null)
                {
                    ColumnHeader header =
                        element.GetContext(typeof (ColumnHeader)) as
                        ColumnHeader;

                    if (header != null)
                    {
                        ClickedColumnHeader = header.Column.Key;

                        // Add check by columns keys for any columns that need to have nonstandard behaviour.
                        if (!isFirstClickPerformed && header.Column.Key == "Value" &&
                            valueIndicator == SortIndicator.None)
                        {
                            isFirstClickPerformed = true;
                            header.Column.SortIndicator = SortIndicator.Descending;
                        }
                        else
                            isFirstClickPerformed = false;
                    }
                }
            }
        }


        /// <summary>
        /// Called when [before sort change].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Infragistics.Win.UltraWinGrid.BeforeSortChangeEventArgs"/> instance containing the event data.</param>
        private void OnBeforeSortChange(object sender, BeforeSortChangeEventArgs e)
        {
            // Add check by columns keys for any columns that need to have nonstandard behaviour.
            if (ClickedColumnHeader == "Value")
            {
                SortedColumnsCollection sortedColumns = e.SortedColumns;
                SortIndicator valueIndicator = grid_.DisplayLayout.Bands[0].Columns[ClickedColumnHeader].SortIndicator;

                // Sort only by column we need.
                if (sortedColumns.Count > 1)
                {
                    for (int i = 0; i < sortedColumns.Count; i++)
                    {
                        UltraGridColumn column = sortedColumns[i];
                        if (column.Key != ClickedColumnHeader)
                            sortedColumns.Remove(column);
                    }
                }

                if (isFirstClickPerformed && sortedColumns.Count > 0 && sortedColumns[0].Key == ClickedColumnHeader &&
                    sortedColumns[0].SortIndicator == SortIndicator.Ascending &&
                    (valueIndicator == SortIndicator.Descending || valueIndicator == SortIndicator.None))
                {
                    e.Cancel = true;
                }
            }
        }
    }
}